// ReflectionMap_Phong effect
// 
// Copyright Outerspace Software, all rights reserved

string description = "A reflectionmapping (spheremap) effect with phong lighting. The 2nd texture can be used as a colourmap.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		LightPos;
float4		EyePos;

float4		AmbientCol;
float4		DiffuseCol;
float4		SpecularCol;
float		Power = 100;

float4		TextureResolution0;
float4		TextureResolution1;

float4 Prop0
<
string UIName="Lighting Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop1
<
string UIName="Reflection Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop2
<
string UIName="Colourmap Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop3
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

texture MyTexture1;
sampler MyTextureSampler1 = sampler_state
{
   Texture = <MyTexture1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
	float4 worldpos:	TEXCOORD1;
	float3 normal:		TEXCOORD3;	
	float2 reflection:	TEXCOORD4;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float3 RotateReflection(float3 a){
	if(Prop3.x==0 && Prop3.y==0 && Prop3.z==0) return a;

	float sa=sin(Prop3.x);
	float ca=cos(Prop3.x);
	float sb=sin(Prop3.y);
	float cb=cos(Prop3.y);
	float sg=sin(Prop3.z);
	float cg=cos(Prop3.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;

	float4 pos = mul(input.position, World);

	output.worldpos 	= pos;
	output.position 	= mul(pos, ViewProjection);
	output.tex		= input.tex;

	float3 NormalVector	= normalize(mul(input.normal,WorldInverseTranspose));
	float3 EyeVector	= normalize(EyePos - pos);




	float3 ReflectionVector = reflect(EyeVector,NormalVector);

	ReflectionVector=RotateReflection(ReflectionVector);

	ReflectionVector.z=abs(ReflectionVector.z);

	ReflectionVector.z+=1;
	float f=2*length(ReflectionVector);
	output.reflection = float2(0.5,0.5) + ReflectionVector.xy/f;

	output.normal = NormalVector;
	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float3 NormalVector		= normalize(input.normal);
	float3 LightVector		= normalize(LightPos - input.worldpos);
	float3 EyeVector		= normalize(EyePos   - input.worldpos);
	float3 LightReflectionVector	= -reflect(LightVector,NormalVector);

	//not lightened
	float4 c=DiffuseCol;

	//Use lighting?
	if(Prop0.x>0){
		//ambient component
		float4 d=AmbientCol;

		//diffuse component
		d += DiffuseCol * saturate(dot(NormalVector, LightVector));

		//specular component
		d += SpecularCol * pow(saturate(dot(EyeVector, LightReflectionVector)),Power);
		
		d.w = DiffuseCol.w;

		d=(d*Prop0.x)+(float4(1,1,1,1)*(1-Prop0.x));
		c*=d;
	}

	//Use first texture as spheremap?
	if(Prop1.x>0 && TextureResolution0.x>0){
		float4 d=DiffuseCol*tex2D(MyTextureSampler0,input.reflection);

		d=(d*Prop1.x)+(float4(1,1,1,1)*(1-Prop1.x));
		c*=d;
	}

	//Use second texture as colourmap?
	if(Prop2.x>0 && TextureResolution1.x>0){
		float4 d=DiffuseCol*tex2D(MyTextureSampler1,input.tex);
		d.xyz *= d.w;

		d=(d*Prop2.x)+(float4(1,1,1,1)*(1-Prop2.x));
		c*=d;
	}

	output.color = c;
	return output;
}

technique Shader
{
    pass P1
    {
        VertexShader	= compile vs_2_0 VS();
        PixelShader	= compile ps_2_0 PS();
    }
}

