// Reflection(Spheremap) & BumpMap(NormalMap) Shader
// 
// Copyright Outerspace Software, all rights reserved

string description = "This effect combines reflectionmapping with bumpmapping. The first texture is used as a reflectionmap (spheremap) and the second as a bumpmap (normalmap).";

float4x4	WorldInverseTranspose;
float4x4	WorldViewProjection;
float4		EyePosModelSpace;
float4		DiffuseCol;
float4		UVBounds0;

float4 Prop0
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

texture MyTexture1;
sampler MyTextureSampler1 = sampler_state
{
   Texture = <MyTexture1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:			TEXCOORD0;
	float3 tangent:		TANGENT;
};

struct VS_OUTPUT{
    float4 position:	POSITION;
    float2 tex:			TEXCOORD0;
    float3 eye:			TEXCOORD1;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float2 Normal2UVBounds(float2 inUV, float4 inUVBounds){
	inUV.x = inUVBounds.x + inUV.x*(inUVBounds.y-inUVBounds.x);
	inUV.y = inUVBounds.z + inUV.y*(inUVBounds.w-inUVBounds.z);
	return inUV;
}

float3 RotateReflection(float3 a){
	if(Prop0.x==0 && Prop0.y==0 && Prop0.z==0) return a;

	float sa=sin(Prop0.x);
	float ca=cos(Prop0.x);
	float sb=sin(Prop0.y);
	float cb=cos(Prop0.y);
	float sg=sin(Prop0.z);
	float cg=cos(Prop0.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;
	output.position = mul(input.position, WorldViewProjection);
	output.tex = input.tex.xy;

	float3 n = normalize(input.normal);

	float3x3 M;
    	M[0] = input.tangent;
    	M[1] = cross(input.tangent, n);
    	M[2] = n;

	float3 Eye	= EyePosModelSpace - input.position;
	output.eye.xyz = mul(M,Eye);

	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float3 bumpNormal	= 2 * (tex2D(MyTextureSampler1, input.tex) - 0.5);
    	bumpNormal.y = -bumpNormal.y; 

	float3 EyeVector  = normalize(input.eye);

	float3 ReflectionVector = reflect(EyeVector,bumpNormal);
	ReflectionVector = mul(ReflectionVector, WorldInverseTranspose);
	
	ReflectionVector=RotateReflection(ReflectionVector);

	ReflectionVector.z=abs(ReflectionVector.z);

	ReflectionVector.z+=1;
	float f=2*length(ReflectionVector);
	float2 t = float2(0.5,0.5) + ReflectionVector.xy/f;

	t=Normal2UVBounds(t,UVBounds0);

	output.color.xyz = tex2D(MyTextureSampler0, t);

	output.color.w=DiffuseCol.w;
	return output;
}


technique Shader
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

