// ReflectionMap / SphereMap Shader with additive Blending
//
// Copyright Outerspace Software, all rights reserved

string description = "A reflectionmapping (spheremap) effect with additive blending. It uses the first texture as a reflectionmap (spheremap).";

//VS_OUTPUT heeft een COLOR prop voor grafische kaarten zonder pixel shader

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldViewProjection;
float4x4	WorldInverseTranspose;
float4		EyePos;
float4		DiffuseCol;
float4		ImageResolution0;
float4		UVBounds0;

bool optimization=false;

float4 Prop0
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
    float4 position:	POSITION;
    float4 diffuse:	COLOR;
    float2 tex:		TEXCOORD0;
};

struct PS_OUTPUT{
    float4 color:	COLOR;
};

float3 RotateReflection(float3 a){
	if(Prop0.x==0 && Prop0.y==0 && Prop0.z==0) return a;

	float sa=sin(Prop0.x);
	float ca=cos(Prop0.x);
	float sb=sin(Prop0.y);
	float cb=cos(Prop0.y);
	float sg=sin(Prop0.z);
	float cg=cos(Prop0.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

float2 Normal2UVBounds(float2 inUV){
	inUV.x = UVBounds0.x + inUV.x*(UVBounds0.y-UVBounds0.x);
	inUV.y = UVBounds0.z + inUV.y*(UVBounds0.w-UVBounds0.z);
	return inUV;
}

float4 FixColour(float4 c){
	c.xyz*=c.w;
	c.w=c.x;
	if(c.y>c.w) c.w=c.y;
	if(c.z>c.w) c.w=c.z;
	return c;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;
	
	float4 Pos=mul(input.position, World);
	output.position = mul(Pos, ViewProjection);
	output.diffuse = FixColour(DiffuseCol);


	float3 NormalVector		= normalize(mul(input.normal,WorldInverseTranspose));
	float3 EyeVector		= normalize(EyePos - Pos);

	float3 ReflectionVector = reflect(EyeVector,NormalVector);

	ReflectionVector=RotateReflection(ReflectionVector);

	ReflectionVector.z=abs(ReflectionVector.z);

	ReflectionVector.z+=1;
	float f=2*length(ReflectionVector);
	output.tex = float2(0.5,0.5) + ReflectionVector.xy/f;

	output.tex=Normal2UVBounds(output.tex);
	
	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=DiffuseCol;
	if(ImageResolution0.x>0) c*=tex2D(MyTextureSampler0, input.tex);

	output.color=FixColour(c);
	return output;
}

technique Shader
{
    pass P0
    {
	Alphablendenable	=TRUE;
	SrcBlend		=ONE;
	DestBlend		=ONE;
	CullMode		=NONE;
	ZWriteEnable		=FALSE;
    
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

