// LED_Blue.fx
//
// Copyright Outerspace Software, all rights reserved

string description="This effect simulates an blue LED screen. Apply this effect to the camera layer.";
 
float4x4	WorldViewProjection;
float4		UVBounds0;
float4		ExportResolution;

float cRedLED		= 0;
float cGreenLED		= 0;
float cBlueLED		= 1;
float cSmooth		= 0.25;
float cHilight		= 0.25;

texture MyTexture0;
sampler Samp = sampler_state{
    Texture   = (MyTexture0);
    AddressU  = CLAMP;
    AddressV  = CLAMP;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

VS_OUTPUT VS(VS_INPUT input){
    VS_OUTPUT output;	
    output.position = mul(input.position,WorldViewProjection);
    output.tex	= input.tex;     
    return output;
}

float4 GetColour(float2 inUV){

    inUV.x=(inUV.x-UVBounds0.x)/(UVBounds0.y-UVBounds0.x);
    inUV.y=(inUV.y-UVBounds0.z)/(UVBounds0.w-UVBounds0.z);

    float  View  =  ExportResolution.z;
    float2 Scale =  float2(1.0 / ExportResolution.x, 1.0 / ExportResolution.y);
	
    float2 Pbase = inUV - fmod(inUV, Scale);
    float2 PCenter = Pbase + Scale*0.5;

    float2 a=inUV-PCenter;
    a.y*=(ExportResolution.y/ExportResolution.x);    
    float d=length(a);

    float r=Scale.x * 0.5;

    float4 c=tex2D(Samp,PCenter);
    float g=0.333*(c.x+c.y+c.z);

    float g2=g*0.9;
    float4 T=float4(0.1+g2*cRedLED,0.1+g2*cGreenLED,0.1+g2*cBlueLED,1);

    if(View>=1) return T;

    d/=r;

    if(d>=1) return float4(0,0,0,1);

    if(d>=(1.0-cSmooth)){
	float f=(1.0-d)/cSmooth;
	return f*T;
    }

    if(d>cHilight || View>=0.25) return T;

    float f=(cHilight-d)/cHilight;
    return T + f*(float4(g,g,g,1)-T);
}

PS_OUTPUT PS(VS_OUTPUT input){
    PS_OUTPUT output;    
    output.color = GetColour(input.tex);	   
    return output;
}

technique pixels{
    pass p0{   
	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS();
    }
}

